{% extends 'base.html' %}
{% load i18n %}

{% block content %}
<div class="hero-unit">
<h1>{%trans "Thank you! your account has been confirmed" %}</h1>
<p>{% trans "You can now login from the login page using your supplied credentials" %}</p>
<p><a href="/login">Login</a></p>
</div>
{% endblock %}
