{% load i18n %}
{%blocktrans%}
Someone has requested an user registration, if it was you please click on the link below to confirm 
your user registration, Thanks!
{%endblocktrans%}
{{SITE_SCHEME}}://{{SITE_URL}}{{url}}

